@@ -3,7 +3,7 @@ use crate::{
     direction::Direction,
     event::{Event, EventResult, Key, MouseButton, MouseEvent},
     rect::Rect,
-    theme::{ColorStyle, Effect},
+    style::PaletteStyle,
     utils::lines::simple::{prefix, simple_prefix, LinesIterator, Row},
     view::{CannotFocus, ScrollBase, SizeCache, View},
     Vec2, {Printer, With, XY},
@@ -97,13 +97,15 @@ impl TextArea {
         self.cursor
     }
 
-    /// Moves the cursor to the given position.
+    /// Moves the cursor to the given byte position.
     ///
     /// # Panics
     ///
-    /// This method panics if `cursor` is not the beginning of a character in
+    /// This method panics if `cursor` is not the starting byte of a character in
     /// the content string.
     pub fn set_cursor(&mut self, cursor: usize) {
+        // TODO: What to do if we fall inside a multi-codepoint grapheme?
+        // Move back to the start of the grapheme?
         self.cursor = cursor;
 
         let focus = self.selected_row();
@@ -286,9 +288,7 @@ impl TextArea {
     // next line. To show that, we need to add a fake "ghost" row, just for
     // the cursor.
     fn fix_ghost_row(&mut self) {
-        if self.rows.is_empty()
-            || self.rows.last().unwrap().end != self.content.len()
-        {
+        if self.rows.is_empty() || self.rows.last().unwrap().end != self.content.len() {
             // Add a fake, empty row at the end.
             self.rows.push(Row {
                 start: self.content.len(),
@@ -394,7 +394,7 @@ impl TextArea {
 
     /// Fix a damage located at the cursor.
     ///
-    /// The only damages are assumed to have occured around the cursor.
+    /// The only damages are assumed to have occurred around the cursor.
     ///
     /// This is an optimization to not re-compute the entire rows when an
     /// insert happened.
@@ -423,8 +423,7 @@ impl TextArea {
         let last_byte = self.content[self.cursor..]
             .find('\n')
             .map(|i| 1 + i + self.cursor);
-        let last_row = last_byte
-            .map_or(self.rows.len(), |last_byte| self.row_at(last_byte));
+        let last_row = last_byte.map_or(self.rows.len(), |last_byte| self.row_at(last_byte));
         let last_byte = last_byte.unwrap_or(self.content.len());
 
         debug!("Content: `{}` (len={})", self.content, self.content.len());
@@ -442,13 +441,11 @@ impl TextArea {
 
         // First attempt, if scrollbase status didn't change.
         debug!("Rows: {:?}", self.rows);
-        let new_rows =
-            make_rows(&self.content[first_byte..last_byte], available);
+        let new_rows = make_rows(&self.content[first_byte..last_byte], available);
         // How much did this add?
         debug!("New rows: {:?}", new_rows);
         debug!("{}-{}", first_row, last_row);
-        let new_row_count =
-            self.rows.len() + new_rows.len() + first_row - last_row;
+        let new_row_count = self.rows.len() + new_rows.len() + first_row - last_row;
         if !scrollable && new_row_count > size.y {
             // We just changed scrollable status.
             // This changes everything.
@@ -461,8 +458,7 @@ impl TextArea {
 
         // Otherwise, replace stuff.
         let affected_rows = first_row..last_row;
-        let replacement_rows =
-            new_rows.into_iter().map(|row| row.shifted(first_byte));
+        let replacement_rows = new_rows.into_iter().map(|row| row.shifted(first_byte));
         self.rows.splice(affected_rows, replacement_rows);
         self.fix_ghost_row();
         self.scrollbase.set_heights(size.y, self.rows.len());
@@ -478,7 +474,7 @@ impl View for TextArea {
         // (we always keep a space at the end)
         // And y = number of rows
         debug!("{:?}", self.rows);
-        let scroll_width = if self.rows.len() > constraint.y { 1 } else { 0 };
+        let scroll_width = usize::from(self.rows.len() > constraint.y);
 
         let content_width = if self.rows.iter().any(|row| row.is_wrapped) {
             // If any row has been wrapped, we want to take the full width.
@@ -491,49 +487,52 @@ impl View for TextArea {
     }
 
     fn draw(&self, printer: &Printer) {
-        printer.with_color(ColorStyle::secondary(), |printer| {
-            let effect = if self.enabled && printer.enabled {
-                Effect::Reverse
-            } else {
-                Effect::Simple
-            };
+        let (style, cursor_style) = if self.enabled && printer.enabled {
+            (PaletteStyle::EditableText, PaletteStyle::EditableTextCursor)
+        } else {
+            (
+                PaletteStyle::EditableTextInactive,
+                PaletteStyle::EditableTextInactive,
+            )
+        };
 
-            let w = if self.scrollbase.scrollable() {
-                printer.size.x.saturating_sub(1)
-            } else {
-                printer.size.x
-            };
-            printer.with_effect(effect, |printer| {
-                for y in 0..printer.size.y {
-                    printer.print_hline((0, y), w, " ");
-                }
+        let w = if self.scrollbase.scrollable() {
+            printer.size.x.saturating_sub(1)
+        } else {
+            printer.size.x
+        };
+        printer.with_style(style, |printer| {
+            for y in 0..printer.size.y {
+                printer.print_hline((0, y), w, " ");
+            }
+        });
+
+        debug!("Content: `{}`", &self.content);
+        self.scrollbase.draw(printer, |printer, i| {
+            debug!("Drawing row {}", i);
+            let row = &self.rows[i];
+            debug!("row: {:?}", row);
+            let text = &self.content[row.start..row.end];
+            debug!("row text: `{}`", text);
+            printer.with_style(style, |printer| {
+                printer.print((0, 0), text);
             });
 
-            debug!("Content: `{}`", &self.content);
-            self.scrollbase.draw(printer, |printer, i| {
-                debug!("Drawing row {}", i);
-                let row = &self.rows[i];
-                debug!("row: {:?}", row);
-                let text = &self.content[row.start..row.end];
-                debug!("row text: `{}`", text);
-                printer.with_effect(effect, |printer| {
-                    printer.print((0, 0), text);
-                });
-
-                if printer.focused && i == self.selected_row() {
-                    let cursor_offset = self.cursor - row.start;
-                    let c = if cursor_offset == text.len() {
-                        "_"
-                    } else {
-                        text[cursor_offset..]
-                            .graphemes(true)
-                            .next()
-                            .expect("Found no char!")
-                    };
-                    let offset = text[..cursor_offset].width();
+            if printer.focused && i == self.selected_row() {
+                let cursor_offset = self.cursor - row.start;
+                let c = if cursor_offset == text.len() {
+                    "_"
+                } else {
+                    text[cursor_offset..]
+                        .graphemes(true)
+                        .next()
+                        .expect("Found no char!")
+                };
+                let offset = text[..cursor_offset].width();
+                printer.with_style(cursor_style, |printer| {
                     printer.print((offset, 0), c);
-                }
-            });
+                });
+            }
         });
     }
 
@@ -547,36 +546,24 @@ impl View for TextArea {
             Event::Char(ch) => self.insert(ch),
             Event::Key(Key::Enter) => self.insert('\n'),
             Event::Key(Key::Backspace) if self.cursor > 0 => self.backspace(),
-            Event::Key(Key::Del) if self.cursor < self.content.len() => {
-                self.delete()
-            }
+            Event::Key(Key::Del) if self.cursor < self.content.len() => self.delete(),
 
             Event::Key(Key::End) => {
                 let row = self.selected_row();
                 self.cursor = self.rows[row].end;
-                if row + 1 < self.rows.len()
-                    && self.cursor == self.rows[row + 1].start
-                {
+                if row + 1 < self.rows.len() && self.cursor == self.rows[row + 1].start {
                     self.move_left();
                 }
             }
             Event::Ctrl(Key::Home) => self.cursor = 0,
             Event::Ctrl(Key::End) => self.cursor = self.content.len(),
-            Event::Key(Key::Home) => {
-                self.cursor = self.rows[self.selected_row()].start
-            }
+            Event::Key(Key::Home) => self.cursor = self.rows[self.selected_row()].start,
             Event::Key(Key::Up) if self.selected_row() > 0 => self.move_up(),
-            Event::Key(Key::Down)
-                if self.selected_row() + 1 < self.rows.len() =>
-            {
-                self.move_down()
-            }
+            Event::Key(Key::Down) if self.selected_row() + 1 < self.rows.len() => self.move_down(),
             Event::Key(Key::PageUp) => self.page_up(),
             Event::Key(Key::PageDown) => self.page_down(),
             Event::Key(Key::Left) if self.cursor > 0 => self.move_left(),
-            Event::Key(Key::Right) if self.cursor < self.content.len() => {
-                self.move_right()
-            }
+            Event::Key(Key::Right) if self.cursor < self.content.len() => self.move_right(),
             Event::Mouse {
                 event: MouseEvent::WheelUp,
                 ..
@@ -597,9 +584,7 @@ impl View for TextArea {
                 offset,
             } if position
                 .checked_sub(offset)
-                .map(|position| {
-                    self.scrollbase.start_drag(position, self.last_size.x)
-                })
+                .map(|position| self.scrollbase.start_drag(position, self.last_size.x))
                 .unwrap_or(false) =>
             {
                 fix_scroll = false;
@@ -617,9 +602,7 @@ impl View for TextArea {
                 event: MouseEvent::Press(_),
                 position,
                 offset,
-            } if !self.rows.is_empty()
-                && position.fits_in_rect(offset, self.last_size) =>
-            {
+            } if !self.rows.is_empty() && position.fits_in_rect(offset, self.last_size) => {
                 if let Some(position) = position.checked_sub(offset) {
                     #[allow(deprecated)]
                     let y = position.y + self.scrollbase.start_line;
@@ -643,10 +626,7 @@ impl View for TextArea {
         EventResult::Consumed(None)
     }
 
-    fn take_focus(
-        &mut self,
-        _: Direction,
-    ) -> Result<EventResult, CannotFocus> {
+    fn take_focus(&mut self, _: Direction) -> Result<EventResult, CannotFocus> {
         self.enabled.then(EventResult::consumed).ok_or(CannotFocus)
     }
 
@@ -669,9 +649,11 @@ impl View for TextArea {
                 .width()
         };
 
-        Rect::from_size(
-            (self.selected_col(), self.selected_row()),
-            (char_width, 1),
-        )
+        Rect::from_size((self.selected_col(), self.selected_row()), (char_width, 1))
     }
 }
+
+#[crate::blueprint(TextArea::new())]
+struct Blueprint {
+    content: Option<String>,
+}
